#!/usr/bin/perl -w

# pabber version 1.1 - in development starting 11/18/2000
# full functionality as originally planned
# added preference file

# pabber is an attempt to add the functionally of the Personal Address
# Book found in Microsoft Exchange/Outlook. pabber will allow a user
# to search the Global Address Book and add users to pine's .addressbook
# file. It does this using LDAP routines that hook it into an Exchange
# server. Obviously, the user will need to have read access to such
# a beast.

# this piece uses the plook code to make a scrolling list of the results

# pabber is Copyright (c) 2000  Marc Richter - marc@vitinc.com
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.


###########################################################################
# set the following variables to your local install

$size_limit = 75; # if you get no results from known good names, try
                  # making this a smaller value

# no more user changeable items...only proceed if you're a code hound <grin>
###########################################################################

$version ="1.11";

use Tk;
require Tk::Dialog;

use Net::LDAP qw(:all);
use Env qw(HOME);    # grab user HOME directory

sub freshen; # need to foward declare
sub update_list;
sub add_entry;
sub failed_add;
sub get_prefs;

# subroutine for search...modify it to meet my needs

    sub LDAPsearch
        {
          my ($ldap,$searchString,$attrs) = @_ ;
          my $result = $ldap->search (
                base    => "$base",
                scope   => "sub",
                filter  => "$searchString",
                attrs   =>  $attrs,
		sizelimit => $size_limit
                );
        }

my @Attrs = ();             # request all available attributes
                                # to be returned.


# gimme some GUI!

$mw = MainWindow->new;

$mw->title("pabber - v$version");

#create menu bar at top

$mbar = $mw->Frame(-width=>200, height =>25, -relief=>'ridge',
		                             -borderwidth=>2)->pack(
					     -fill=>'x',
					     -side=>'top');
$mbar->Menubutton(-text => "File",
		  -tearoff => 0,
		-menuitems => [[ 'command' => "Add Entry to .addressbook",
				 -command => \&add_entry],
			       [ 'command' => "Show Preferences...",
				 -command => \&pref_dialog],
			       [ 'command' => "Set Preferences...",
				 -command => \&set_prefs],
			       [ 'command' => "Exit",
				 -command =>sub{exit}]])->pack(
					      -side => 'left',
					     
					      -anchor => 'w'
					      );

$mbar->Menubutton(-text => "Help",
		  -tearoff => 0,
		-menuitems => [ [ 'command' => "About pabber...",
				 -command => \&about_box]])->pack(
				              -side =>'left',
					     			  
					      -anchor => 'w'
					      );

# define search entry widget and appropriate search button

my $search_string = $mw->Entry()->pack(-side =>"top",-anchor=>'w');

$mw->Button(-text => "Find",
	    -command => sub {update_list($search_string->get());})->
	    pack(-anchor=>'w');

# define listbox widget that is used to list search results
# also, bind clicks on entries to "freshen" sub, which
# will update the details for the selected entry
# so that we may review the item before we add to .addressbook

my $lb = $mw->Scrolled("Listbox",-width=>'40', -scrollbars => "osoe")->pack(-anchor=>'w');

$lb->bind("<Button-1>", sub {freshen;});

#define display fields for selected user in search list

$mw->Label(-text=>"E-mail address")->pack(-anchor=>'w');
my $email_address = $mw->Entry(-width=>'40')->pack(-anchor=>'w');
$mw->Label(-text=>"Phone")->pack(-anchor=>'w');
my $phone = $mw->Entry(-width=>'20')->pack(-anchor=>'w');
$mw->Label(-text=>"Office")->pack(-anchor=>'w');
my $office = $mw->Entry(-width=>'20')->pack(-anchor=>'w');

# the "Add" button

$mw->Button(-text =>"Add to .addressbook",
	    -command => sub {add_entry;})->pack (-anchor=>'w');

get_prefs;  # read preferences from $HOME\.pabrc
MainLoop;

sub freshen
{
    @selected = $lb->curselection;
    foreach (@selected) {
	my $mail_displayed= $entries[$_]->get_value('mail');
	my $phone_displayed= $entries[$_]->get_value('telephoneNumber');
	my $office_displayed= $entries[$_]->
	    get_value('physicalDeliveryOfficeName');
	$email_address->delete(0,'end');
        $email_address->insert(0,$mail_displayed);
	$phone->delete(0,'end');
	$phone->insert(0,$phone_displayed);
	$office->delete(0,'end');
	$office->insert(0,$office_displayed);
    }
}

sub update_list

# most of the "heavy lifting" takes place here, including attaching
# to the serving and calling the search routine

{
    my $gettem = $_[0];
 #   print $gettem;
    $lb->delete(0, 'end');	
    $email_address->delete(0,'end');
    $phone->delete(0,'end');
# the following bit was added for 1.01 - stops time-out problem
    undef $ldap;                          
    $ldap = Net::LDAP->new($server);
# ok, check to see if we connected
    if (! defined $ldap) {
	$mw->Dialog(-text=>"Unable to attach\nto server\n$server",
		    -title=>"Error",
		    -default_button=>'OK',
		    -buttons=>[qw\OK\])->Show;
		} else {   
		    $result = LDAPsearch($ldap,"sn=" . $gettem,\@Attrs);
		    @entries = $result->entries;  
		    foreach $entr (@entries)
		    {
			$lb->insert("end", $entr->get_value('cn'));
		    }
		    undef @selected;     # reset selected so we can't add 
		                         # an item
                                         # until we click on it in the list
		}
}


sub add_entry {
    if (! defined @selected) {
	$mw->Dialog(-text =>"Please select an\nentry to add\n",
		    -title=>"Error",
		    -default_button=>'OK',
		    -buttons =>[qw/OK/])->Show;
    }
    else {
	open (PAB, ">>$HOME/.addressbook") or failed_add;
	foreach (@selected) {
	    $mail_a= $entries[$_]->get_value('mail');
	    $nick_a= $entries[$_]->get_value('rdn');  # at Merck, this is the
	                                             # NT username
	    $name_a= $entries[$_]->get_value('cn');   # full name
	}
	print PAB ($nick_a, "\t", $name_a, "\t", $mail_a, "\n");
	close (PAB);
	$mw->Dialog(-text =>"Entry added for\n$name_a\n",
		    -title=>"Entry Added",
		    -default_button=>'OK',
		    -buttons =>[qw/OK/])->Show;
    }
}

sub failed_add {
    $mw->Dialog(-text =>"Failed to open\n$HOME\.addressbook",
		-title =>"Error",
		-default_button=>'OK',
		-buttons =>[qw/OK/])->Show;
    die;
}

sub pref_dialog {
    $mw->Dialog(-text =>"current server: $server\ncurrent base:\n$base\n",
		-title =>'Preferences',
		-default_button=>'OK',
		-buttons => [qw/OK/])->Show;
}

sub about_box {
    $mw->Dialog(-text =>"pabber is released\nunder the GPL\nCopyright 2000\nby Marc Richter\nmarc\@vitinc.com\nversion $version", -title =>'About pabber',
		      -default_button=>'OK', -buttons =>[qw/OK/])->Show;
}


sub get_prefs {
    open (PABRC, "$HOME/.pabrc") or $FAILED=1;
    if ($FAILED) {
	print "No $HOME/.pabrc found. Creating default.\n";
	$server="myserver.myplace.com";
	$base="c=us";
	write_prefs;
    } else {
	$server=<PABRC>;
	chomp($server);
	$base=<PABRC>;
	chomp($base);
	close (PABRC);
    }
}

sub write_prefs {
    open (PABRC, ">$HOME/.pabrc") or die "unable to open $HOME/.pabrc";
    print PABRC ($server,"\n",$base,"\n");
    close PABRC;
}

sub set_prefs {
    if (! Exists($prefs)){
	$prefs = $mw->Toplevel();
	$prefs->title("Set Preferences");
	$prefs->Label(-text=>"Server")->pack(-anchor=>'w');
	$prefs->Entry(-width=>'40', -textvariable=>\$server)->
	    pack(-anchor=>'w');
	$prefs->Label(-text=>"Base")->pack(-anchor=>'w');
	$prefs->Entry(-width=>'40', -textvariable=>\$base)->
	    pack(-anchor=>'w');
	$prefs->Label(-text=>"note: c=us should work for folk in the USA")->pack;
	$prefs->Button(-text=>"OK",
		       -command=>sub {write_prefs;
				      $prefs->withdraw;})->pack;
	$prefs->Button(-text=>"Cancel",
		       -command=>sub {$prefs->withdraw;})->pack;
    } else {
	$prefs->deiconify();
	$prefs->raise();
    }
}
